@@ -98,6 +98,7 @@ item_in_dir () {
 	else
 		q=""
 	fi
+	[ -d "$2" ] || return 1
 	# find files with any case
 	ls -1 "$2" | grep $q -i "^$1$"
 }
@@ -117,7 +118,9 @@ fs_type () {
 
 parse_proc_mounts () {
 	while read -r line; do
+		set -f
 		set -- $line
+		set +f
 		printf '%s %s %s\n' "$(mapdevfs "$1")" "$2" "$3"
 	done
 }
@@ -129,7 +132,9 @@ parsefstab () {
 				:	
 			;;
 			*)
+				set -f
 				set -- $line
+				set +f
 				printf '%s %s %s\n' "$1" "$2" "$3"
 			;;
 		esac
@@ -191,7 +196,9 @@ linux_mount_boot () {
 		# Try to mount any /boot partition.
 		bootmnt=$(parsefstab < "$tmpmnt/etc/fstab" | grep " /boot ") || true
 		if [ -n "$bootmnt" ]; then
+			set -f
 			set -- $bootmnt
+			set +f
 			boottomnt=""
 
 			# Try to map labels and UUIDs ourselves if possible,
@@ -252,11 +259,16 @@ linux_mount_boot () {
 					debug "found boot partition $1 for linux system on $partition, but cannot map to existing device"
 				else
 					debug "found boot partition $bootpart for linux system on $partition"
-					ro_partition "$boottomnt"
-					if mount -o ro "$boottomnt" "$tmpmnt/boot" -t "$3"; then
+					if which grub-mount >/dev/null 2>&1 && \
+					   grub-mount "$boottomnt" "$tmpmnt/boot" 2>/dev/null; then
 						mounted=1
 					else
-						error "failed to mount $boottomnt on $tmpmnt/boot"
+						ro_partition "$boottomnt"
+						if mount -o ro "$boottomnt" "$tmpmnt/boot" -t "$3"; then
+							mounted=1
+						else
+							error "failed to mount $boottomnt on $tmpmnt/boot"
+						fi
 					fi
 				fi
 			fi
